/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef MHPoint_H
#define MHPoint_H

#include <MHString.h>
#include <MHVector.h>



/**
*  A class for storing chart data points or other numerical
*  data that needs to be stored in a time serie.
*  There are also some static utility methods to convert time series of the points.
*/
class MHPoint : public MH {
public:
    enum {
                            MVG_ATR           =  14,
                            MVG_BOLLINGER     =  20,
                            MVG_LONG          =  16,
                            MVG_MOMENTUM      =  10,
                            MVG_RSI           =  14,
                            MVG_SHORT         =   8,
                            MVG_STD           =  20,
                            MVG_STOCHASTICS_D =   3,
                            MVG_STOCHASTICS_K =   5,
                            SORT_DATE,
    };

                            MHPoint ();
                            MHPoint (const MHPoint&);
                            MHPoint (const double x, const double y);
                            MHPoint (const char* d, const double y = 0);
                            MHPoint (const char* d, const double h, const double l, const double c);
                            MHPoint (const char* d, const double h, const double l, const double c, const double o, const double v);

    MHPoint&                operator=(const MHPoint&);
    bool                    operator==(const MHPoint& x) {return (strcmp (Date, x.Date) == 0) ? true : false;}
    bool                    operator==(const char* s) {return (strcmp (Date, s) == 0) ? true : false;}
    bool                    operator!=(const MHPoint& x) {return (strcmp (Date, x.Date) != 0) ? true : false;}
    bool                    operator>(const MHPoint& x) {return (strcmp (Date, x.Date) > 0) ? true : false;}
    bool                    operator>(const char* s) {return (strcmp (Date, s) > 0) ? true : false;}
    bool                    operator<(const MHPoint& x) {return (strcmp (Date, x.Date) < 0) ? true : false;}
    bool                    operator<(const char* s){return (strcmp (Date, s) < 0) ? true : false;}
    bool                    operator<=(const MHPoint& x) {return (strcmp (Date, x.Date) < 1) ? true : false;}
    bool                    operator<=(const char* x) {return (strcmp (Date, x) < 1) ? true : false;}
    bool                    operator>=(const MHPoint& x) {return (strcmp (Date, x.Date) >= 0) ? true : false;}
    bool                    operator>=(const char* x) {return (strcmp (Date, x) >= 0) ? true : false;}

    void                    Adjust (double price, double volume = -1);
    static void             ATR (MHVector* in, MHVector*out, int nMvg);
    static void             Bollinger (MHVector* in, MHVector* mvg, MHVector* upper, MHVector* lower, int nDays);
    const char*             Class () {return "MHPoint";}
    static void             Clear (MHVector*);
    int                     Compare (const MH* o, int type){return strcmp(Date, ((MHPoint*)o)->Date);}
    static void             Copy (MHVector* in, MHVector* out, const char* pStartDate = 0, const char* pStopDate = 0);
    static void             CopyPointers (MHVector* in, MHVector* out);
    static void             Cut (MHVector* in, MHVector* out);
    static void             Day2Week (MHVector* in, MHVector* out, int nWeekDay);
    static void             Momentum (MHVector* in, MHVector*out, MHVector* zero, int nMvg);
    static void             MovingAverage (MHVector* in, MHVector* out, int nMvg);
    static void             Normalise (MHVector* in, MHVector* out, double nOrigo);
    static void             Print (MHVector* in);
    static void             RSI (MHVector* in, MHVector*out, int nMvg);
    static void             RSI2 (MHVector* in, MHVector*out, int nMvg);
    static bool             Save (const char* pFilename, MHVector* in);
    static void             StdDev (MHVector* in, MHVector* out, int nDays);
    static void             Stochastics (MHVector* in, MHVector* out, int nMvg);
    void                    Validate ();
    static void             Volume (MHVector* in, MHVector* out);

    double                  GetClose() {return C;}
    const char*             GetDate() {return Date;}
    double                  GetHigh() {return H;}
    double                  GetLow() {return L;}
    double                  GetOpen() {return O;}
    double                  GetX() {return X;}
    double                  GetVolume() {return V;}

    void                    SetClose(double val) {C = val;}
    void                    SetDate (const char* val) {strncpy (Date, val, 8); Date[8] = '\0';}
    void                    SetDate (const MHString* val) {strncpy (Date, val->Get(), 8); Date[8] = '\0';}
    void                    SetHigh(double val) {H = val;}
    void                    SetLow(double val) {L = val;}
    void                    SetOpen(double val) {O = val;}
    void                    SetX(double val) {X = val;}
    void                    SetVolume(double val) {V = val;}

protected:
    double                  X;
    double                  H;
    double                  L;
    double                  C;
    double                  O;
    double                  V;
    char                    Date[9];
};



/**
*  Create new object with everything set to 0
*/
inline MHPoint::MHPoint () : MH() {
    X = H = L = C = O = V = 0;
}



/**
*  Create new point with x and y values
*  @param double& - X value
*  @param double& - Y value
*/
inline MHPoint::MHPoint (const double x, const double y) : MH() {
    X = x;
    L = C = V = H = O = y;
}



/**
*  Create new point with date and y value
*  @param const char* - X value in the form of a date string
*  @param double      - Y value
*/
inline MHPoint::MHPoint (const char* d, const double y) : MH() {
    STRNCPY (Date, d, 8);
    X = L = C = O = H = y;
    V = -1;
}



/**
*  Create new price point
*  @param char*   - X value in the form of a date
*  @param double& - Highest value in this data point
*  @param double& - Lowest value in this data point
*  @param double& - Last value in this data point
*/
inline MHPoint::MHPoint (const char* d, const double h, const double l, const double c) : MH() {
    STRNCPY (Date, d, 8);
    X = -1;
    H = h;
    L = l;
    C = c;
    O = V = -1;
}



/**
*  Operator =
*  @param const MHPoint& - Right side of expression
*/
inline MHPoint& MHPoint::operator=(const MHPoint& x) {
    strncpy (Date, x.Date, 8);
    Date[8] = '\0';
    X = x.X;
    H = x.H;
    L = x.L;
    C = x.C;
    O = x.O;
    V = x.V;
    return *this;
}



/**
*  Adjust price and volume
*  If volume is less than 0 then the volume is divided with the price factor
*/
inline void MHPoint::Adjust (double price, double volume) {
    if (price > 0.0000001) {
        H *= price;
        L *= price;
        C *= price;
        O *= price;
    }
    if (volume > 0.0000001)
        V *= volume;
    else
        V /= price;
}

#endif
