/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <MHUtil.h>
#include <MHDate.h>
#include <MHFileTransaction.h>
#include <minilzo.h>

#define HEAP_ALLOC(var,size) \
lzo_align_t __LZO_MMODEL var [ ((size) + (sizeof(lzo_align_t) - 1)) / sizeof(lzo_align_t) ]
static HEAP_ALLOC(wrkmem,LZO1X_1_MEM_COMPRESS);
int MHUtil::aaInitCompress = 0;

/**
*  Create a pointer to char
*  @param int     - Rows
*  @param int     - Size fo row
*  @return char** - Allocated string, remember to delete later
*/
char** MHUtil::Allocate (int rows, int size) {
    char** s = new char*[rows];

    for (int f = 0; f < rows; f++) {
        s[f]    = new char[size];
        s[f][0] = '\0';
    }
    return s;
}



/**
*  Allocate new buffer
*  @param char*   - Old string
*  @param int     - Size
*  @return char*  - New string
*/
char* MHUtil::Allocate (char* old, int& size) {
    char* nw = new char[size * 2];

    strcpy (nw, old);
    delete []old;
    size = size * 2;
    return nw;
}



/**
*  Append buffer to another buffer
*  @param char*       - String 1
*  @param const char* - String 2
*  @param int         - Size of destination buffer
*  @return char*      - Destination string
*/
char* MHUtil::AppendString (char* s1, const char* s2, int& size) {
    if (s1 && s2) {
        int len1 = STRLEN (s1);
        int len2 = STRLEN (s2);
        int tot  = len1 + len2;

        if ((tot + 1) >= size) {
            char* tmp = new char [tot * 2];

            strcpy (tmp, s1);
            strcat (tmp, s2);
            delete []s1;
            s1 = tmp;
            size = tot * 2;
        }
        else {
            strcat (s1, s2);
        }
    }
    return s1;
}



/**
*  Append buffer to another buffer
*  @param char*       - String 1
*  @param const       - String 2
*  @param int         - Size of destination buffer
*  @return char*      - Destination string
*/
char* MHUtil::AppendChar (char* s1, char s2, int& size) {
    if (s1) {
        int len1 = STRLEN (s1);

        if ((len1 + 1) >= size) {
            char* tmp = new char [len1 * 2];

            strcpy (tmp, s1);
            tmp[len1] = s2;
            tmp[len1 + 1] = '\0';
            delete []s1;
            s1 = tmp;
            size = len1 * 2;
        }
        else {
            s1[len1]     = s2;
            s1[len1 + 1] = '\0';;
        }
    }
    return s1;
}



/**
*  Get current market price of a bond.
*  @param double  - Interest value
*  @param double  - Interest rate, in percent
*  @param double  - Maturity value of bond
*  @param int     - Years to maturity
*  @return double - Value
*/
double MHUtil::BondValue (double interest_value, double interest, double maturity, int years_maturity) {
    double value = 0;

    interest /= 100;
    for (int f = 1; f <= years_maturity; f++) {
        value += interest_value / (pow (1.0 + interest, f));
    }
    value += maturity / (pow (1.0 + interest, years_maturity));
    return value;
}



/**
*  Return change in percent
*  @param double  - Start number
*  @param double  - Stop number
*  @return double - Change in percent
*/
double MHUtil::Change (double start, double stop) {
    double diff = stop - start;

    if (fabs (start) > 0.0000000001)
        return (diff / start) * 100.0;
    else
        return 0;
}



/**
*  Count all \n in a file
*  @param const char* - Filename
*  @return int        - Number of rows
*/
int MHUtil::CountRowsInFile (const char* fileName) {
    char*   buffer  = new char[100000];
    FILE*   file    = fopen (fileName, "rb");
    int     rows    = 0;

    if (file) {
        int bytes = fread (buffer, sizeof(char), 100000, file);

        while (bytes > 0) {
            for (int f = 0; f < bytes; f++) {
                if (buffer[f] == '\n')
                    rows++;
            }
            bytes = fread (buffer, sizeof(char), 100000, file);
        }
        fclose(file);
    }
    delete []buffer;

    return rows;
}



/**
*  Delete a pointer to char
*  @param char**  - String
*  @param int     - Rows
*/
void MHUtil::Delete (char** s, int rows) {
    for (int f = 0; f < rows; f++)
        delete s[f];
    delete []s;
}



/**
*  Does file exist, 0 bytes or more
*  @param const char* - Filename
*  @return int        - Size in bytes
*/
bool MHUtil::FileExist (const char* fileName) {
    struct stat s;
    if (stat (fileName, &s) == 0)
        return true;
    else
        return false;
}



/**
*  Get size of file
*  @param const char* - Filename
*  @return int        - Size in bytes
*/
int MHUtil::FileSize (const char* fileName) {
    struct stat s;
    if (stat (fileName, &s) == 0)
        return s.st_size;
    else
        return 0;
}



/**
*  Return interest.
*  @param const char* - Start date
*  @param const char* - Stop date
*  @param double      - Interest / year in percent
*  @return double     - Interest between start and stop date
*/
double MHUtil::Interest (const char* d1, const char* d2, double nInterest) {
    double years = MHDate::NoYears (d1, d2);

    nInterest /= 100;
    nInterest += 1.0;
    nInterest = pow (nInterest, years);
    nInterest -= 1.0;
    return (nInterest * 100);
}



/**
*  Return interst.
*  @param const char* - Start date
*  @param const char* - Stop date
*  @param double      - Amount
*  @param double      - Interest / year in percent
*  @return double     - Interest between start and stop date
*/
double MHUtil::Interest (const char* d1, const char* d2, double nValue, double nInterest) {
    double interest = MHUtil::Interest (d1, d2, nInterest);
    return MHUtil::ChangeVal (nValue, interest);
}



/**
*  Return change in percent
*  @param double  - First number
*  @param double  - Second number
*  @return double - Percent (second/first) 10, 5 = 200
*/
double MHUtil::PercentOf (double a, double b) {
    if (fabs (b) > 0.0000000001)
        return (a / b) * 100;
    else
        return 0;
}



/**
*  Read file from disk. If compressed with lzo uncompress it
*  @param const char* - Filename
*  @param int&        - Filesize
*  @return char*      - New buffer with file data
*/
char* MHUtil::ReadFile (const char* fileName, int& size) {
    MHString    tmpName;
    char*       buffer = 0;
    FILE*       file   = 0;

    if (!aaInitCompress) {
        lzo_init();
        aaInitCompress = 1;
    }


    MHFileTransaction::GetFileName (fileName, &tmpName, true);

    size        = MHUtil::FileSize (tmpName());
    buffer      = new char[size + 8];
    file        = fopen (tmpName(), "rb");
    buffer[0]   = '\0';

    if (size > 0) {
        buffer[size]     = '\0';
        buffer[size + 1] = '\0';
    }

    if (file) {
        int bytes = fread (buffer, sizeof(char), size, file);
        fclose (file);
        if (bytes != size) {
            delete []buffer;
            throw "Can't open file";
        }

        if (memcmp (buffer, "lzo\0", 4) == 0) {
            lzo_uint  cmpSize, cmpSize2;
            memcpy (&cmpSize, buffer + 4, sizeof (cmpSize));
            char* buff = new char[cmpSize + cmpSize / 64 + 16 + 3];

            if (lzo1x_decompress ((unsigned char*)buffer + 8, size - 8, (unsigned char*)buff, &cmpSize2, wrkmem) != LZO_E_OK) {
                delete []buff;
                throw "Can't read file";
            }
            if (cmpSize != cmpSize2) {
                delete []buff;
                throw "Can't read file";
            }
            delete []buffer;
            buffer = buff;
            size = cmpSize2;
            return buffer;
        }
        else
            return buffer;
    }
    delete []buffer;
    return 0;
}



/**
*  Save buffer to disk, compresed or not
*/
void MHUtil::WriteFile (const char* fileName, const char* buffer, int size, bool compress) {
    MHString tmpName1;
    MHString tmpName2;

    MHFileTransaction::GetFileName (fileName, &tmpName1, false);

    if (!aaInitCompress) {
        lzo_init();
        aaInitCompress = 1;
    }

    tmpName2 = tmpName1;
    tmpName2 += ".tmp";
    FILE* file  = fopen (tmpName2(), "wb");

    if (file) {
        lzo_uint bytes;
        if (compress) {
            lzo_byte* buff    = new lzo_byte[size + size / 64 + 16 + 3];
            lzo_uint  cmpSize = 0;

            if (lzo1x_1_compress ((unsigned char*)buffer, size, buff, &cmpSize, wrkmem) != LZO_E_OK) {
                delete []buff;
                fclose (file);
                throw "Can't compress file";
            }
            else {
                bytes =  fwrite ("lzo\0", sizeof(char), 4, file);
                bytes += fwrite (&size, sizeof(char), sizeof(size), file);
                bytes += fwrite (buff, sizeof(char), cmpSize, file);
                delete []buff;
                fflush (file);
                fclose (file);
                if (bytes != (cmpSize + 8)) throw "Can't save file to disk";
            }
        }
        else {
            bytes = fwrite (buffer, sizeof(char), size, file);
            fflush (file);
            fclose (file);
            if (bytes != (unsigned int)size) throw "Can't save file to disk";
        }

        MHString tmpName3;
        tmpName3 = tmpName2;
        tmpName3 += ".tmp";
        rename (tmpName1(), tmpName3());
        if (rename (tmpName2(), tmpName1())) {
            //  Failed, move original file back
            rename (tmpName3(), tmpName1());
            remove (tmpName2());
        }
        else
            remove (tmpName3());
        return;
    }
    else
        throw "Can't open file in write mode";
}
