/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.com
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef Fl_DC_h
#define Fl_DC_h

#include <MHUtil.h>
#include <MHDC.h>
#include <FL/fl_draw.H>
#include <FL/Fl.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Scrollbar.H>
#include <FL/Fl_Box.h>
#include <FL/Fl_Positioner.h>



/**
*  Fl_DC is the device context for MHChart to be used with the fltk toolkit.
*/
class Fl_DC : public MHDC, public Fl_Group {
public:
    enum {
                            SCROLL_SIZE = 20,
                            SCROLL_HEIGHT = 16,
                            SPACE = 2,
    };

                            Fl_DC (int x, int y, int w, int h, char *label = 0);
    virtual                 ~Fl_DC ();

    void                    draw ();
    int                     handle (int event);
    void                    resize (int x1, int y1, int x2, int y2);

    void                    BeginClip (int x, int y, int w, int h);
    void                    Color (int color);
    void                    DrawLine (int x1, int y1, int x2, int y2);
    void                    DrawRect (int x1, int y1, int x2, int y2);
    void                    DrawText (const char* p, int x1, int y1);
    void                    EndClip ();
    int                     GetHeight ();
    int                     GetWidth ();
    int                     GetCharHeight ();
    Fl_Scrollbar*           GetScrollbar ();
    int                     GetStringWidth (const char* pString);
    void                    Paint ();
    void                    Recalc (bool bInit, int w = -1, int h = -1);
    bool                    QueryLeft (int x, int y, MHPoint* p);
    bool                    QueryRight (int x, int y, MHPoint* p);
    void                    SetBrushColor (int Color);
    void                    SetPenColor (int Color);
    void                    SetScrollPos (int Pos);
    void                    SetScrollRange ();

private:
    Fl_Box*                 aCanvas;
    Fl_Scrollbar*           aScrollbar;
};



/**
*  Create a clip rectangle
*  @param int - Start x pos
*  @param int - Start y pos
*  @param int - End x pos
*  @param int - End y pos
*/
inline void Fl_DC::BeginClip (int x, int y, int w, int h) {
    fl_push_clip (x + this->x(), y + this->y(), w, h);
}



/**
*  Set new background color
*  @param int - Color
*/
inline void Fl_DC::Color (int color) {
    if (color > 0 && color < 255)
        aCanvas->color (Fl_Color(color));
    else
        aCanvas->color (FL_WHITE);
}



/**
*  Call MHChart::Paint method to paint all charts
*/
inline void Fl_DC::draw () {
    Paint ();
}



/**
*  Draw a line between two points
*  @param int - Start x pos
*  @param int - Start y pos
*  @param int - End x pos
*  @param int - End y pos
*/
inline void Fl_DC::DrawLine (int x1, int y1, int x2, int y2) {
    fl_line (x1 + x(), y1 + y(), x2 + x(), y2 + y());
}



/**
*  Draw a filled rectangle
*  @param int - Start x pos
*  @param int - Start y pos
*  @param int - End x pos
*  @param int - End y pos
*/
inline void Fl_DC::DrawRect (int x, int y, int w, int h) {
    fl_rectf (x + this->x() + 1, y + this->y() + 1, w, h);
}



/**
*  Draw text at specific point
*  @param const char* - String to draw
*  @param int  - Start x pos
*  @param int  - Start y pos
*/
inline void Fl_DC::DrawText (const char* p, int x, int y) {
    fl_font (FL_COURIER, GetFontSize());
    fl_draw (p, x + this->x(), y + this->y());
}



/**
*  Delete a clip rectangle
*/
inline void Fl_DC::EndClip () {
    fl_pop_clip();
}



/**
*  Get height of drawing canvas
*  @return int - Height of window in pixels
*/
inline int Fl_DC::GetHeight () {
    return h();
}



/**
*  Get width of drawing canvas
*  @return int - Width of window in pixels
*/
inline int Fl_DC::GetWidth () {
    return w();
}



/**
*  Get height of current font
*  @return int - Height of char in pixels
*/
inline int Fl_DC::GetCharHeight() {
    return GetFontSize();
}



/**
*  Get pointer to scrollbar
*  @return Fl_Scrollbar - Scrollbar that controls chart
*/
inline Fl_Scrollbar* Fl_DC::GetScrollbar() {
    return aScrollbar;
}



/**
*  Get width in pixels of supplied string
*  @param char* - String to draw
*  @return int - String width in pixels
*/
inline int Fl_DC::GetStringWidth(const char* pString) {
    fl_font (FL_COURIER, GetFontSize());
    return int(fl_width(pString));
}



/**
*  Draw chart
*/
inline void Fl_DC::Paint () {
    Fl_Group::draw();
    MHDC::Paint ();
}



/**
*  Query data point
*  @param int      - X pos
*  @param int      - Y Pos
*  @param MHPoint& - Result point
*  @return bool    - true if found
*/
inline bool Fl_DC::QueryLeft (int x1, int y1, MHPoint* p) {
    return MHDC::QueryLeft (x1 - x(), y1 - y(), p);
}



/**
*  Query data point
*  @param int      - X pos
*  @param int      - Y Pos
*  @param MHPoint& - Result point
*  @return bool    - true if found
*/
inline bool Fl_DC::QueryRight (int x1, int y1, MHPoint* p) {
    return MHDC::QueryRight (x1 - x(), y1 - y(), p);
}

#endif
